# OpenCodeGraph extension for CodeMirror

The [`@opencodegraph/codemirror-extension`](https://www.npmjs.com/package/@opencodegraph/codemirror-extension) npm package implements a [CodeMirror](https://codemirror.net/) extension that shows [OpenCodeGraph](https://opencodegraph.org) annotations in the editor.

Check out the [OpenCodeGraph playground](https://opencodegraph.org/playground) for a live example of this extension.

## Usage

Install it:

```shell
npm install @opencodegraph/codemirror-extension @opencodegraph/client
```

Set up an OpenCodeGraph client and fetch annotations:

```typescript
import { createClient } from '@opencodegraph/client'

// Set up a client.
const client = createClient({
  configuration: () =>
    Promise.resolve({
      enable: true,
      providers: {
        'https://opencodegraph.org/npm/@opencodegraph/provider-hello-world': true,
      },
    }),
  makeRange: r => r,
  logger: console.error,
})

// Fetch annotations for the file.
const annotations = client.annotations({ file: 'file:///foo.js', content: 'my file\nhello\nworld' })
```

Then register the extension with CodeMirror.

If you're using React, the `useOpenCodeGraphExtension` hook makes it easy:

```typescript
import { useOpenCodeGraphExtension } from '@opencodegraph/codemirror-extension'

function MyComponent() {
  // ...

  // A helpful React hook if using React.
  const ocgExtension = useOpenCodeGraphExtension({
    visibility: true,
    annotations,
  })

  // Pass `ocgExtension` to CodeMirror as an extension.

  // ...
}
```

Otherwise, set up the extension manually. You'll need to implement your own decoration (instead of `<OpenCodeGraphItemChipList>` shown here) if not using React.

```tsx
import type { Extension } from '@codemirror/state'
import { openCodeGraphData, showOpenCodeGraphDecorations } from '@opencodegraph/codemirror-extension'

const ocgExtension: Extension = [
  openCodeGraphData(annotations),
  showOpenCodeGraphDecorations({
    visibility: true,
    createDecoration(container, { indent, items }) {
      const root = createRoot(container)
      root.render(
        <Indented indent={indent}>
          <OpenCodeGraphItemChipList items={items} chipClassName="ocg-chip" popoverClassName="ocg-chip-popover" />
        </Indented>
      )
      return {
        destroy() {
          root.unmount()
        },
      }
    },
  }),
]

// Pass `ocgExtension` to CodeMirror as an extension.
```
