import type { Extension } from '@codemirror/state'
import { type Annotation } from '@opencodegraph/client'
import { OpenCodeGraphItemChipList } from '@opencodegraph/ui-react'
import { useMemo, type FunctionComponent, type ReactNode } from 'react'
import { createRoot } from 'react-dom/client'
import { openCodeGraphData, showOpenCodeGraphDecorations } from './extension'

export function useOpenCodeGraphExtension({
    visibility,
    annotations,
}: {
    visibility: boolean
    annotations: Annotation[] | undefined
}): Extension {
    // TODO(sqs): useCompartment results in sometimes the facet data being stale because the editor is reconfigured
    return useMemo(
        () =>
            visibility
                ? [
                      openCodeGraphData(annotations),
                      showOpenCodeGraphDecorations({
                          visibility,
                          createDecoration(container, { indent, items }) {
                              const root = createRoot(container)
                              root.render(
                                  <Indented indent={indent}>
                                      <OpenCodeGraphItemChipList
                                          items={items}
                                          chipClassName="ocg-chip"
                                          popoverClassName="ocg-chip-popover"
                                      />
                                  </Indented>
                              )
                              return {
                                  destroy() {
                                      root.unmount()
                                  },
                              }
                          },
                      }),
                  ]
                : [],
        [visibility, annotations]
    )
}

/**
 * Indents the children by the specified whitespace. This is used to align OCG item chips with the
 * non-whitespace characters on the line they're attached to.
 */
const Indented: FunctionComponent<{ indent: string | undefined; children: ReactNode }> = ({ indent, children }) => (
    // eslint-disable-next-line react/forbid-dom-props
    <div style={{ display: 'flex' }}>
        {/* eslint-disable-next-line react/forbid-dom-props */}
        <pre style={{ height: 0, overflow: 'hidden', visibility: 'hidden' }}>{indent}</pre>
        {children}
    </div>
)
